@@ -29,7 +29,7 @@
 #include "nojobs.c"
 #else /* JOB_CONTROL */
 
-#include <sys/types.h>
+#include "bashtypes.h"
 #include "trap.h"
 #include <stdio.h>
 #include <signal.h>
@@ -46,9 +46,9 @@
 #endif /* _POSIX_VERSION */
 
 #include <sys/file.h>
+#include "filecntl.h"
 #include <sys/ioctl.h>
 #include <sys/param.h>
-#include "filecntl.h"
 
 #if defined (BUFFERED_INPUT)
 #  include "input.h"
@@ -64,7 +64,7 @@
 #if defined (_POSIX_VERSION) && !defined (TERMIOS_MISSING)
 #  undef NEW_TTY_DRIVER
 #  define TERMIOS_TTY_DRIVER
-#  if defined (sun)
+#  if defined (sun) && !defined (_POSIX_SOURCE)	/* XXX - SunOS4, SunOS5? */
 #    define _POSIX_SOURCE
 #  endif
 #else /* !_POSIX_VERSION */
@@ -87,18 +87,18 @@
 #if defined (TERMIOS_TTY_DRIVER)
 /* For Sun workstations we undefine a couple of defines so that
    the inclusion of termios.h won't cause complaints. */
-#  if defined (sun)
+#  if defined (SunOS4)
 #    undef ECHO
 #    undef NOFLSH
 #    undef TOSTOP
-#  endif /* sun */
+#  endif /* SunOS4 */
 #  include <termios.h>
 #endif /* TERMIOS_TTY_DRIVER */
 
 /* For the TIOCGPGRP and TIOCSPGRP ioctl parameters on HP-UX */
 
 #if defined (hpux) && !defined (TERMIOS_TTY_DRIVER)
-#include <bsdtty.h>
+#  include <bsdtty.h>
 #endif /* hpux && !TERMIOS_TTY_DRIVER */
 
 #include "bashansi.h"
@@ -113,9 +113,15 @@
 extern int errno;
 #endif /* !errno */
 
+/* Variables used here but defined in other files. */
 extern int interactive, interactive_shell, asynchronous_notification;
-extern char *shell_name;
-extern sighandler sigint_sighandler ();
+extern int subshell_environment;
+extern int posixly_correct, no_symbolic_links, shell_level;
+extern int interrupt_immediately, last_command_exit_value;
+extern int loop_level, breaking;
+extern Function *this_shell_builtin;
+extern char *shell_name, *this_command_name;
+extern sigset_t top_level_mask;
 
 /* The array of known jobs. */
 JOB **jobs = (JOB **)NULL;
@@ -226,7 +232,7 @@ sigprocmask (operation, newset, oldset)
       break;
 
     default:
-      report_error ("Bad code in jobs.c: sigprocmask");
+      internal_error ("Bad code in jobs.c: sigprocmask");
     }
 
   if (oldset)
@@ -241,15 +247,12 @@ sigprocmask (operation, newset, oldset)
 static char *
 current_working_directory ()
 {
-  extern char *getwd ();
-  extern char *the_current_working_directory;
-  extern int follow_symbolic_links;
   char *dir;
   static char d[MAXPATHLEN];
 
   dir = get_string_value ("PWD");
 
-  if (!dir && the_current_working_directory && !follow_symbolic_links)
+  if (!dir && the_current_working_directory && no_symbolic_links)
     dir = the_current_working_directory;
 
   if (!dir)
@@ -292,12 +295,14 @@ making_children ()
   start_pipeline ();
 }
 
+void
 stop_making_children ()
 {
   already_making_children = 0;
 }
 
 /* Start building a pipeline.  */
+void
 start_pipeline ()
 {
   if (the_pipeline)
@@ -314,7 +319,7 @@ start_pipeline ()
   if (job_control)
     {
       if (pipe (pgrp_pipe) == -1)
-	report_error ("start_pipeline: pgrp pipe");
+	internal_error ("start_pipeline: pgrp pipe");
     }
 #endif
 }
@@ -330,7 +335,6 @@ stop_pipeline (async, deferred)
 {
   register int i, j;
   JOB *newjob = (JOB *)NULL;
-  char *get_string_value ();
   sigset_t set, oset;
 
   BLOCK_CHILD (set, oset);
@@ -374,8 +378,8 @@ stop_pipeline (async, deferred)
   /* Do we need more room? */
   if (i == job_slots)
     {
-      jobs = (JOB **)xrealloc
-	(jobs, (1 + (job_slots += JOB_SLOTS)) * sizeof (JOB *));
+      job_slots += JOB_SLOTS;
+      jobs = (JOB **)xrealloc (jobs, ((1 + job_slots) * sizeof (JOB *)));
 
       for (j = i; j < job_slots; j++)
 	jobs[j] = (JOB *)NULL;
@@ -488,15 +492,16 @@ cleanup_dead_jobs ()
 
 /* Delete the job at INDEX from the job list.  Must be called
    with SIGCHLD blocked. */
-delete_job (index)
-     int index;
+void
+delete_job (job_index)
+     int job_index;
 {
-  register JOB *temp = jobs[index];
+  register JOB *temp = jobs[job_index];
 
-  if (index == current_job || index == previous_job)
+  if (job_index == current_job || job_index == previous_job)
     reset_current ();
 
-  jobs[index] = (JOB *)NULL;
+  jobs[job_index] = (JOB *)NULL;
 
   free (temp->wd);
   discard_pipeline (temp->pipe);
@@ -549,11 +554,13 @@ add_process (name, pid)
     {
       register PROCESS *p = t->next;
 
-      while (p->next != t->next) p = p->next;
+      while (p->next != t->next)
+	p = p->next;
       p->next = t;
     }
 }
 
+#if 0
 /* Take the last job and make it the first job.  Must be called with
    SIGCHLD blocked. */
 rotate_the_pipeline ()
@@ -580,13 +587,14 @@ reverse_the_pipeline ()
     ;
   p->next = (PROCESS *)NULL;
 
-  n = (PROCESS *)reverse_list (the_pipeline);
+  n = REVERSE_LIST (the_pipeline, PROCESS *);
 
   the_pipeline = n;
   for (p = the_pipeline; p->next; p = p->next)
     ;
   p->next = the_pipeline;
 }
+#endif
 
 /* Map FUNC over the list of jobs.  If FUNC returns non-zero,
    then it is time to stop mapping, and that is the return value
@@ -595,6 +603,7 @@ reverse_the_pipeline ()
 static int
 map_over_jobs (func, arg1, arg2)
      Function *func;
+     int arg1, arg2;
 {
   register int i;
   int result;
@@ -618,6 +627,7 @@ map_over_jobs (func, arg1, arg2)
 }
 
 /* Cause all the jobs in the current pipeline to exit. */
+void
 terminate_current_pipeline ()
 {
   if (pipeline_pgrp && pipeline_pgrp != shell_pgrp)
@@ -660,6 +670,7 @@ hangup_all_jobs ()
     }
 }
 
+void
 kill_current_pipeline ()
 {
   stop_making_children ();
@@ -728,6 +739,7 @@ find_job (pid)
 }
 
 /* Print descriptive information about the job with leader pid PID. */
+void
 describe_pid (pid)
      pid_t pid;
 {
@@ -763,8 +775,8 @@ describe_pid (pid)
 	Use format JLIST_NORMAL, but list only jobs about which
 	the user has not been notified. */
 static void
-pretty_print_job (index, format, stream)
-     int index, format;
+pretty_print_job (job_index, format, stream)
+     int job_index, format;
      FILE *stream;
 {
   register PROCESS *p, *first, *last;
@@ -777,14 +789,14 @@ pretty_print_job (index, format, stream)
   /* Format only pid information about the process group leader? */
   if (format == JLIST_PID_ONLY)
     {
-      fprintf (stream, "%d\n", jobs[index]->pipe->pid);
+      fprintf (stream, "%d\n", jobs[job_index]->pipe->pid);
       UNBLOCK_CHILD (oset);
       return;
     }
 
   if (format == JLIST_CHANGED_ONLY)
     {
-      if (jobs[index]->flags & J_NOTIFIED)
+      if (jobs[job_index]->flags & J_NOTIFIED)
 	{
 	  UNBLOCK_CHILD (oset);
 	  return;
@@ -792,17 +804,17 @@ pretty_print_job (index, format, stream)
       format = JLIST_STANDARD;
     }
 
-  fprintf (stream, "[%d]%c ", index + 1,
-	   (index == current_job) ? '+':
-	   (index == previous_job) ? '-' : ' ');
+  fprintf (stream, "[%d]%c ", job_index + 1,
+	   (job_index == current_job) ? '+':
+	   (job_index == previous_job) ? '-' : ' ');
 
-  first = last = p = jobs[index]->pipe;
+  first = last = p = jobs[job_index]->pipe;
   while (last->next != first)
     last = last->next;
 
   /* We have printed information about this job.  When the job's
      status changes, flush_child () sets the notification flag to 0. */
-  jobs[index]->flags |= J_NOTIFIED;
+  jobs[job_index]->flags |= J_NOTIFIED;
 
   for (;;)
     {
@@ -819,20 +831,19 @@ pretty_print_job (index, format, stream)
 	  PROCESS *show = format ? p : last;
 	  char *temp = "Done";
 
-	  if (JOBSTATE (index) == JSTOPPED && !format)
+	  if (JOBSTATE (job_index) == JSTOPPED && !format)
 	    temp = "Stopped";
 
-	  if (JOBSTATE (index) == JRUNNING)
+	  if (JOBSTATE (job_index) == JRUNNING)
 	    temp = "Running";
 	  else
 	    {
 	      if (WIFSTOPPED (show->status))
-		temp = sys_siglist[WSTOPSIG (show->status)];
+		temp = strsignal (WSTOPSIG (show->status));
 	      else if (WIFSIGNALED (show->status))
-		temp = sys_siglist[WTERMSIG (show->status)];
+		temp = strsignal (WTERMSIG (show->status));
 	      else if (WIFEXITED (show->status))
 		{
-		  extern int posixly_correct;
 		  int exit_status;
 
 		  temp = retcode_name_buffer;
@@ -863,16 +874,17 @@ pretty_print_job (index, format, stream)
 
 	  if (temp)
 	    {
+	      int templ = strlen (temp);
 	      fprintf (stream, "%s", temp);
 
-	      if (strlen (temp))
-		name_padding = LONGEST_SIGNAL_DESC - strlen (temp);
+	      if (templ)
+		name_padding = LONGEST_SIGNAL_DESC - templ;
 	      else
 		name_padding = LONGEST_SIGNAL_DESC - 2; /* strlen ("| ") */
 
 	      fprintf (stream, "%*s", name_padding, "");
 
-	      if ((!WIFSTOPPED (show->status)) && (WIFCORED (show->status)))
+	      if ((WIFSTOPPED (show->status) == 0) && (WIFCORED (show->status)))
 		fprintf (stream, "(core dumped) ");
 	    }
 	}
@@ -880,19 +892,20 @@ pretty_print_job (index, format, stream)
       if (p != first && format)
 	fprintf (stream, "| ");
 
-      fprintf (stream, "%s", p->command);
+      if (p->command)
+	fprintf (stream, "%s", p->command);
 
       if (p == last) 
 	{
 	  char *wd = current_working_directory ();
 
-	  if (JOBSTATE (index) == JRUNNING &&
-	      !(jobs[index]->flags & J_FOREGROUND))
+	  if (JOBSTATE (job_index) == JRUNNING &&
+	      !(jobs[job_index]->flags & J_FOREGROUND))
 	    fprintf (stream, " &");
 
-	  if (strcmp (wd, jobs[index]->wd) != 0)
+	  if (strcmp (wd, jobs[job_index]->wd) != 0)
 	    fprintf (stream,
-		     "  (wd: %s)", polite_directory_format (jobs[index]->wd));
+		     "  (wd: %s)", polite_directory_format (jobs[job_index]->wd));
 	}
 
       if (format || (p == last))
@@ -908,16 +921,17 @@ pretty_print_job (index, format, stream)
 }
 
 int
-list_one_job (job, format, ignore, index)
+list_one_job (job, format, ignore, job_index)
      JOB *job;
-     int format, ignore, index;
+     int format, ignore, job_index;
 {
-  pretty_print_job (index, format, stdout);
+  pretty_print_job (job_index, format, stdout);
   return (0);
 }
 
 /* List jobs.  If FORMAT is non-zero, then the long form of the information
    is printed, else just a short version. */
+void
 list_jobs (format)
      int format;
 {
@@ -958,7 +972,7 @@ make_child (command, async_p)
   /* Create the child, handle severe errors. */
   if ((pid = fork ()) < 0)
     {
-      report_error ("fork: %s", strerror (errno));
+      internal_error ("fork: %s", strerror (errno));
 
       /* Kill all of the processes in the current pipeline. */
       terminate_current_pipeline ();
@@ -974,8 +988,6 @@ make_child (command, async_p)
     {
       /* In the child.  Give this child the right process group, set the
 	 signals to the default state for a new process. */
-      extern sigset_t top_level_mask;
-
       pid_t mine = getpid ();
 
 #if defined (BUFFERED_INPUT)
@@ -1052,12 +1064,6 @@ make_child (command, async_p)
 	  signal (SIGTSTP, SIG_DFL);
 	  signal (SIGTTOU, SIG_DFL);
 	  signal (SIGTTIN, SIG_DFL);
-
-	  if (async_p)
-	    {
-	      signal (SIGINT, SIG_IGN);
-	      signal (SIGQUIT, SIG_IGN);
-	    }
 	}
 
 #if defined (PGRP_PIPE)
@@ -1195,16 +1201,18 @@ get_tty_state ()
 #if defined (TERMIOS_TTY_DRIVER)
       if (tcgetattr (tty, &shell_tty_info) < 0)
 	{
-	  extern int shell_level;
-
+#if 0
 	  /* Only print an error message if we're really interactive at
 	     this time. */
 	  if (interactive)
 	    internal_error ("[%d: %d] tcgetattr: %s",
 			    getpid (), shell_level, strerror (errno));
+#endif
+	  return -1;
 	}
 #endif /* TERMIOS_TTY_DRIVER */
     }
+  return 0;
 }
 
 /* Make the current tty use the state in shell_tty_info. */
@@ -1230,16 +1238,16 @@ set_tty_state ()
 #if defined (TERMIOS_TTY_DRIVER)
       if (tcsetattr (tty, TCSADRAIN, &shell_tty_info) < 0)
 	{
-	  extern int shell_level;
-
 	  /* Only print an error message if we're really interactive at
 	     this time. */
 	  if (interactive)
 	    internal_error ("[%d: %d] tcsetattr: %s",
 			    getpid (), shell_level, strerror (errno));
+	  return -1;
 	}
 #endif /* TERMIOS_TTY_DRIVER */
     }
+  return 0;
 }
 
 /* Given an index into the jobs array JOB, return the pid of the last
@@ -1347,13 +1355,9 @@ static sighandler
 wait_sigint_handler (sig)
      int sig;
 {
-  extern Function *this_shell_builtin;
-  extern int interrupt_immediately;
-
   if (interrupt_immediately ||
       (this_shell_builtin && this_shell_builtin == wait_builtin))
     {
-      extern int last_command_exit_value;
       last_command_exit_value = EXECUTION_FAILURE;
       restore_sigint_handler ();
       interrupt_state++;
@@ -1386,7 +1390,6 @@ int
 wait_for (pid)
      pid_t pid;
 {
-  extern int last_command_exit_value;
   int job, termination_state;
   register PROCESS *child;
   sigset_t set, oset;
@@ -1506,11 +1509,11 @@ wait_for (pid)
      interactive.  Clean up any dead jobs in either case. */
   if (job != NO_JOB)
     {
-      if (interactive_shell)
+      if (interactive_shell && !subshell_environment)
 	{
 	  if (WIFSIGNALED (child->status) || WIFSTOPPED (child->status))
 	    set_tty_state ();
-          else
+	  else
 	    get_tty_state ();
 
 	  notify_and_cleanup ();
@@ -1548,6 +1551,7 @@ wait_for_job (job)
    of known jobs.  This does not actually delete jobs when the
    shell is not interactive, because the dead jobs are not marked
    as notified. */
+void
 notify_and_cleanup ()
 {
   if (interactive)
@@ -1558,6 +1562,7 @@ notify_and_cleanup ()
 
 /* Make dead jobs disappear from the jobs array without notification.
    This is used when the shell is not interactive. */
+void
 reap_dead_jobs ()
 {
   mark_dead_jobs_as_notified ();
@@ -1740,8 +1745,6 @@ start_job (job, foreground)
 
   if (!foreground && already_running)
     {
-      extern char *this_command_name;
-
       report_error ("%s: bg background job?", this_command_name);
       UNBLOCK_CHILD (oset);
       return (-1);
@@ -1768,7 +1771,8 @@ start_job (job, foreground)
   do
     {
       fprintf (stderr, "%s%s",
-	       p->command, p->next != jobs[job]->pipe? " | " : "");
+	       p->command ? p->command : "",
+	       p->next != jobs[job]->pipe? " | " : "");
       p = p->next;
     }
   while (p != jobs[job]->pipe);
@@ -1984,7 +1988,7 @@ flush_child (sig)
 		   {
 		      job_state |= child->running;
 		      if (!child->running)
-			any_stopped |= (WIFSTOPPED (child->status));
+			any_stopped |= WIFSTOPPED (child->status);
 		      child = child->next;
 		    }
 		  while (child != jobs[job]->pipe);
@@ -1993,7 +1997,6 @@ flush_child (sig)
 		    {
 		      if (any_stopped)
 			{
-			  extern int loop_level, breaking;
 			  jobs[job]->state = JSTOPPED;
 			  jobs[job]->flags &= ~J_FOREGROUND;
 			  call_set_current++;
@@ -2015,8 +2018,6 @@ flush_child (sig)
 			  if ((WTERMSIG (jobs[job]->pipe->status) == SIGINT) &&
 			      (jobs[job]->flags & J_FOREGROUND))
 			    {
-			      extern int last_command_exit_value;
-
 			      /* If SIGINT is trapped, set the exit status so
 				 that the trap handler can see it. */
 			      if (signal_is_trapped (SIGINT))
@@ -2094,10 +2095,6 @@ flush_child (sig)
   if (job_control && signal_is_trapped (SIGCHLD) &&
       trap_list[SIGCHLD] != (char *)IGNORE_SIG)
     {
-      extern void maybe_set_sigchld_trap ();
-      extern int last_command_exit_value, interrupt_immediately;
-      extern char *trap_list[];
-
       char *trap_command;
 
       /* Turn off the trap list during the call to parse_and_execute ()
@@ -2115,7 +2112,7 @@ flush_child (sig)
       /* We have to add the commands this way because they will be run
 	 in reverse order of adding.  We don't want maybe_set_sigchld_trap ()
 	 to reference freed memory. */
-      add_unwind_protect ((Function *)vfree, trap_command);
+      add_unwind_protect ((Function *)xfree, trap_command);
       add_unwind_protect ((Function *)maybe_set_sigchld_trap, trap_command);
 
       the_pipeline = (PROCESS *)NULL;
@@ -2123,7 +2120,7 @@ flush_child (sig)
       while (children_exited--)
 	{
 	  interrupt_immediately = 1;
-	  parse_and_execute (savestring (trap_command), "trap");
+	  parse_and_execute (savestring (trap_command), "trap", -1);
 	}
 
       run_unwind_frame ("SIGCHLD trap");
@@ -2190,7 +2187,7 @@ notify_of_job_status ()
 		{
 		  if (termsig && WIFSIGNALED (s) && termsig != SIGINT)
 		    {
-		      fprintf (stderr, "%s", sys_siglist[termsig]);
+		      fprintf (stderr, "%s", strsignal (termsig));
 
 		      if (WIFCORED (s))
 			fprintf (stderr, " (core dumped)");
@@ -2298,15 +2295,16 @@ initialize_jobs ()
 	  }
       }
 
-#if defined (NeXT) || defined (alliant)
-      /* Compensate for a bug in the NeXT 2.0 /usr/etc/rlogind. */
+#if defined (RLOGIN_PGRP_BUG)
+      /* Compensate for a bug in systems that compiled the BSD
+	 /usr/etc/rlogind with DEBUG defined, like NeXT and Alliant. */
       if (shell_pgrp == 0)
 	{
 	  shell_pgrp = getpid ();
 	  setpgid (0, shell_pgrp);
 	  tcsetpgrp (shell_tty, shell_pgrp);
 	}
-#endif /* NeXT || alliant */
+#endif /* RLOGIN_PGRP_BUG */
 
       while ((terminal_pgrp = tcgetpgrp (shell_tty)) != -1)
 	{
@@ -2338,8 +2336,11 @@ initialize_jobs ()
 	    }
 
 	  job_control = 1;
-	  give_terminal_to (shell_pgrp);
+	  if (give_terminal_to (shell_pgrp) < 0)		/* XXX */
+	    /* job_control = 0 */;				/* XXX */
 	}
+      if (job_control == 0)
+        internal_error ("no job control in this shell");	/* XXX */
     }
 
   if (shell_tty != fileno (stdin))
@@ -2347,10 +2348,11 @@ initialize_jobs ()
 
   set_signal_handler (SIGCHLD, flush_child);
 
-  change_flag_char ('m', job_control ? '-' : '+');
+  change_flag ('m', job_control ? '-' : '+');
 
   if (interactive)
     get_tty_state ();
+  return job_control;
 }
 
 /* Set the line discipline to the best this system has to offer.
@@ -2391,7 +2393,7 @@ set_new_line_discipline (tty)
 #endif /* TERMIO_TTY_DRIVER */
 
 #if defined (TERMIOS_TTY_DRIVER)
-#if defined (Ultrix) || defined (SunOS4) || defined (aixpc)
+#  if defined (TERMIOS_LDISC)
   if (tcgetattr (tty, &shell_tty_info) < 0)
     return (-1);
 
@@ -2401,7 +2403,7 @@ set_new_line_discipline (tty)
       if (tcsetattr (tty, TCSADRAIN, &shell_tty_info) < 0)
 	return (-1);
     }
-#endif /* Ultrix || SunOS4 || aixpc */
+#  endif /* TERMIOS_LDISC */
   return (0);
 #endif /* TERMIOS_TTY_DRIVER */
 
@@ -2410,19 +2412,6 @@ set_new_line_discipline (tty)
 #endif
 }
 
-/* Allow or disallow job control to take place.  Returns the old value
-   of job_control. */
-int
-set_job_control (arg)
-     int arg;
-{
-  int old;
-
-  old = job_control;
-  job_control = arg;
-  return (old);
-}
-
 static SigHandler *old_tstp, *old_ttou, *old_ttin;
 static SigHandler *old_cont = (SigHandler *)SIG_DFL;
 static sighandler stop_signal_handler (), cont_signal_handler ();
@@ -2451,6 +2440,7 @@ sigwinch_sighandler (sig)
 #endif /* !READLINE && TIOCGWINSZ && SIGWINCH */
 
 /* Setup this shell to handle C-C, etc. */
+void
 initialize_job_signals ()
 {
   if (interactive)
@@ -2512,6 +2502,7 @@ give_terminal_to (pgrp)
      pid_t pgrp;
 {
   sigset_t set, oset;
+  int r = 0;
 
   if (job_control)
     {
@@ -2528,12 +2519,15 @@ give_terminal_to (pgrp)
 	  /* Maybe we should print an error message? */
 /*	  internal_error ("tcsetpgrp(%d) failed: pid %d to pgrp %d: %s",
 	    shell_tty, getpid(), pgrp, strerror (errno)); */
+	  r = -1;
 	}
       else
 	terminal_pgrp = pgrp;
 
       sigprocmask (SIG_SETMASK, &oset, (sigset_t *)NULL);
     }
+
+  return r;
 }
 
 /* Clear out any jobs in the job array.  This is intended to be used by
@@ -2586,8 +2580,22 @@ mark_dead_jobs_as_notified ()
     }
 }
 
+/* Allow or disallow job control to take place.  Returns the old value
+   of job_control. */
+int
+set_job_control (arg)
+     int arg;
+{
+  int old;
+
+  old = job_control;
+  job_control = arg;
+  return (old);
+}
+
 /* Turn off all traces of job control.  This is run by children of the shell
    which are going to do shellsy things, like wait (), etc. */
+void
 without_job_control ()
 {
   stop_making_children ();
