@@ -19,6 +19,7 @@
    is generally kept in a file called COPYING or LICENSE.  If you do not
    have a copy of the license, write to the Free Software Foundation,
    675 Mass Ave, Cambridge, MA 02139, USA. */
+#define READLINE_LIBRARY
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -43,17 +44,19 @@
 extern int errno;
 #endif /* !errno */
 
-/* These next are for filename completion.  Perhaps this belongs
-   in a different place. */
 #include <pwd.h>
-#if defined (USG) && !defined (isc386) && !defined (sgi)
-extern struct passwd *getpwuid (), *getpwent ();
-#endif
-#if defined (isc386) && !defined (__STDC__) && defined (_POSIX_SOURCE)
+#if defined (USG) && !defined (HAVE_GETPW_DECLS)
 extern struct passwd *getpwent ();
-#endif
+#endif /* USG && !HAVE_GETPW_DECLS */
 
-/* #define HACK_TERMCAP_MOTION */
+/* ISC systems don't define getpwent() if _POSIX_SOURCE is defined. */
+#if defined (isc386) && defined (_POSIX_SOURCE)
+#  if defined (__STDC__)
+extern struct passwd *getpwent (void);
+#  else
+extern struct passwd *getpwent ();
+#  endif /* !__STDC__ */
+#endif /* isc386 && _POSIX_SOURCE */
 
 #include "posixstat.h"
 
@@ -95,12 +98,13 @@ extern char *xmalloc (), *xrealloc ();
 /* If non-zero, then this is the address of a function to call when
    completing on a directory name.  The function is called with
    the address of a string (the current directory name) as an arg. */
-Function *rl_symbolic_link_hook = (Function *)NULL;
+Function *rl_directory_completion_hook = (Function *)NULL;
 
 /* Non-zero means readline completion functions perform tilde expansion. */
 int rl_complete_with_tilde_expansion = 0;
 
-#define VISIBLE_STATS
+/* If non-zero, non-unique completions always show the list of matches. */
+int _rl_complete_show_all = 0;
 
 #if defined (VISIBLE_STATS)
 #  if !defined (X_OK)
@@ -135,6 +139,10 @@ Function *rl_completion_entry_function = (Function *)NULL;
    array of strings returned. */
 CPPFunction *rl_attempted_completion_function = (CPPFunction *)NULL;
 
+/* Non-zero means to suppress normal filename completion after the
+   user-specified completion function has been called. */
+int rl_attempted_completion_over = 0;
+
 /* Local variable states what happened during the last completion attempt. */
 static int completion_changed_buffer = 0;
 
@@ -146,22 +154,24 @@ rl_complete (ignore, invoking_key)
      int ignore, invoking_key;
 {
   if (rl_last_func == rl_complete && !completion_changed_buffer)
-    rl_complete_internal ('?');
+    return (rl_complete_internal ('?'));
+  else if (_rl_complete_show_all)
+    return (rl_complete_internal ('!'));
   else
-    rl_complete_internal (TAB);
+    return (rl_complete_internal (TAB));
 }
 
 /* List the possible completions.  See description of rl_complete (). */
 rl_possible_completions (ignore, invoking_key)
      int ignore, invoking_key;
 {
-  rl_complete_internal ('?');
+  return (rl_complete_internal ('?'));
 }
 
 rl_insert_completions (ignore, invoking_key)
      int ignore, invoking_key;
 {
-  rl_complete_internal ('*');
+  return (rl_complete_internal ('*'));
 }
 
 /* The user must press "y" or "n". Non-zero return means "y" pressed. */
@@ -172,9 +182,9 @@ get_y_or_n ()
   for (;;)
     {
       c = rl_read_key ();
-      if (c == 'y' || c == 'Y')
+      if (c == 'y' || c == 'Y' || c == ' ')
 	return (1);
-      if (c == 'n' || c == 'N')
+      if (c == 'n' || c == 'N' || c == RUBOUT)
 	return (0);
       if (c == ABORT_CHAR)
 	rl_abort ();
@@ -265,11 +275,90 @@ _delete_quotes (text)
 }
 #endif /* SHELL */
 
+/* Return the portion of PATHNAME that should be output when listing
+   possible completions.  If we are hacking filename completion, we
+   are only interested in the basename, the portion following the
+   final slash.  Otherwise, we return what we were passed. */
+static char *
+printable_part (pathname)
+      char *pathname;
+{
+  char *temp = (char *)NULL;
+
+  if (rl_filename_completion_desired)
+    temp = strrchr (pathname, '/');
+
+  if (!temp)
+    return (pathname);
+  else
+    return (++temp);
+}
+
+/* Output TO_PRINT to rl_outstream.  If VISIBLE_STATS is defined and we
+   are using it, check for and output a single character for `special'
+   filenames.  Return 1 if we printed an extension character, 0 if not. */
+static int
+print_filename (to_print, full_pathname)
+     char *to_print, *full_pathname;
+{
+#if !defined (VISIBLE_STATS)
+  fputs (to_print, rl_outstream);
+  return 0;
+#else  
+  char *s, c, *new_full_pathname;
+  int extension_char = 0, slen, tlen;
+
+  fputs (to_print, rl_outstream);
+  if (rl_filename_completion_desired && rl_visible_stats)
+    {
+      /* If to_print != full_pathname, to_print is the basename of the
+	 path passed.  In this case, we try to expand the directory
+	 name before checking for the stat character. */
+      if (to_print != full_pathname)
+	{
+	  /* Terminate the directory name. */
+	  c = to_print[-1];
+	  to_print[-1] = '\0';
+
+	  s = tilde_expand (full_pathname);
+	  if (rl_directory_completion_hook)
+	    (*rl_directory_completion_hook) (&s);
+
+	  slen = strlen (s);
+	  tlen = strlen (to_print);
+	  new_full_pathname = xmalloc (slen + tlen + 2);
+	  strcpy (new_full_pathname, s);
+	  new_full_pathname[slen] = '/';
+	  strcpy (new_full_pathname + slen + 1, to_print);
+
+	  extension_char = stat_char (new_full_pathname);
+
+	  free (new_full_pathname);
+	  to_print[-1] = c;
+	}
+      else
+	{
+	  s = tilde_expand (full_pathname);
+	  extension_char = stat_char (s);
+	}
+
+      free (s);
+      if (extension_char)
+	putc (extension_char, rl_outstream);
+      return (extension_char != 0);
+    }
+  else
+    return 0;
+#endif /* VISIBLE_STATS */
+}
+
 /* Complete the word at or before point.
    WHAT_TO_DO says what to do with the completion.
    `?' means list the possible completions.
    TAB means do standard completion.
-   `*' means insert all of the possible completions. */
+   `*' means insert all of the possible completions.
+   `!' means to do standard completion, and list all possible completions if
+   there is more than one. */
 rl_complete_internal (what_to_do)
      int what_to_do;
 {
@@ -278,10 +367,8 @@ rl_complete_internal (what_to_do)
   int start, scan, end, delimiter = 0, pass_next;
   char *text, *saved_line_buffer;
   char *replacement;
-#if defined (SHELL)
   char quote_char = '\0';
   int found_quote = 0;
-#endif
 
   if (rl_line_buffer)
     saved_line_buffer = savestring (rl_line_buffer);
@@ -318,6 +405,7 @@ rl_complete_internal (what_to_do)
 	      if (rl_line_buffer[scan] == '\\')
 		{
 		  pass_next = 1;
+		  found_quote |= 4;
 		  continue;
 		}
 
@@ -336,12 +424,11 @@ rl_complete_internal (what_to_do)
 		  /* Found start of a quoted substring. */
 		  quote_char = rl_line_buffer[scan];
 		  rl_point = scan + 1;
-#if defined (SHELL)
+		  /* Shell-like quoting conventions. */
 		  if (quote_char == '\'')
 		    found_quote |= 1;
 		  else if (quote_char == '"')
 		    found_quote |= 2;
-#endif
 		}
 	    }
 	}
@@ -349,42 +436,41 @@ rl_complete_internal (what_to_do)
       if (rl_point == end)
 	{
 	  int quoted = 0;
-	  /* We didn't find an unclosed quoted substring up which to do
+	  /* We didn't find an unclosed quoted substring upon which to do
 	     completion, so use the word break characters to find the
 	     substring on which to complete. */
 	  while (--rl_point)
 	    {
+	      scan = rl_line_buffer[rl_point];
+
+	      if (strchr (rl_completer_word_break_characters, scan) == 0)
+		continue;
+
 #if defined (SHELL)
 	      /* Don't let word break characters in quoted substrings break
 		 words for the completer. */
-	      if (found_quote)
-		{
-		  if (strchr (rl_completer_quote_characters, rl_line_buffer[rl_point]))
-		    {
-		      quoted = !quoted;
-		      continue;
-		    }
-		  if (quoted)
-		    continue;
-		}
+	      if (found_quote && char_is_quoted (rl_line_buffer, rl_point))
+		continue;
 #endif /* SHELL */
-	      if (strchr (rl_completer_word_break_characters, rl_line_buffer[rl_point]))
-	        break;
+
+	      /* Convoluted code, but it avoids an n2 algorithm with calls
+	      	 to char_is_quoted. */
+	      break;
 	    }
 	}
 
       /* If we are at a word break, then advance past it. */
-      if (strchr (rl_completer_word_break_characters, rl_line_buffer[rl_point]))
+      scan = rl_line_buffer[rl_point];
+      if (strchr (rl_completer_word_break_characters, scan))
 	{
 	  /* If the character that caused the word break was a quoting
 	     character, then remember it as the delimiter. */
-	  if (strchr ("\"'", rl_line_buffer[rl_point]) && (end - rl_point) > 1)
-	    delimiter = rl_line_buffer[rl_point];
+	  if (strchr ("\"'", scan) && (end - rl_point) > 1)
+	    delimiter = scan;
 
 	  /* If the character isn't needed to determine something special
 	     about what kind of completion to perform, then advance past it. */
-	  if (!rl_special_prefixes ||
-	      !strchr (rl_special_prefixes, rl_line_buffer[rl_point]))
+	  if (!rl_special_prefixes || strchr (rl_special_prefixes, scan) == 0)
 	    rl_point++;
 	}
     }
@@ -401,11 +487,9 @@ rl_complete_internal (what_to_do)
     {
       matches = (*rl_attempted_completion_function) (text, start, end);
 
-      if (matches)
+      if (matches || rl_attempted_completion_over)
 	{
-	  /* XXX - This is questionable code. - XXX */
-	  if (matches == (char **)-1)
-	    matches = (char **)NULL;
+	  rl_attempted_completion_over = 0;
 	  our_func = (Function *)NULL;
 	  goto after_usual_completion;
 	}
@@ -434,6 +518,7 @@ rl_complete_internal (what_to_do)
   else
     {
       register int i;
+      int should_quote;
 
       /* It seems to me that in all the cases we handle we would like
 	 to ignore duplicate possiblilities.  Scan for the text to
@@ -443,6 +528,7 @@ rl_complete_internal (what_to_do)
 	  char *lowest_common;
 	  int j, newlen = 0;
 	  char dead_slot;
+	  char **temp_array;
 
 	  /* Sort the items. */
 	  /* It is safe to sort this array, because the lowest common
@@ -466,25 +552,19 @@ rl_complete_internal (what_to_do)
 
 	  /* We have marked all the dead slots with (char *)&dead_slot.
 	     Copy all the non-dead entries into a new array. */
-	  {
-	    char **temp_array =
-	      (char **)xmalloc ((3 + newlen) * sizeof (char *));
+	  temp_array = (char **)xmalloc ((3 + newlen) * sizeof (char *));
+	  for (i = j = 1; matches[i]; i++)
+	    {
+	      if (matches[i] != (char *)&dead_slot)
+		temp_array[j++] = matches[i];
+	    }
+	  temp_array[j] = (char *)NULL;
 
-	    for (i = 1, j = 1; matches[i]; i++)
-	      {
-		if (matches[i] != (char *)&dead_slot)
-		  temp_array[j++] = matches[i];
-	      }
+	  if (matches[0] != (char *)&dead_slot)
+	    free (matches[0]);
+	  free (matches);
 
-	    temp_array[j] = (char *)NULL;
-
-	    if (matches[0] != (char *)&dead_slot)
-	      free (matches[0]);
-
-	    free (matches);
-
-	    matches = temp_array;
-	  }
+	  matches = temp_array;
 
 	  /* Place the lowest common denominator back in [0]. */
 	  matches[0] = lowest_common;
@@ -502,6 +582,7 @@ rl_complete_internal (what_to_do)
       switch (what_to_do)
 	{
 	case TAB:
+	case '!':
 	  /* If we are matching filenames, then here is our chance to
 	     do clever processing by re-examining the list.  Call the
 	     ignore function with the array as a parameter.  It can
@@ -520,8 +601,16 @@ rl_complete_internal (what_to_do)
 	     matches don't require a quoted substring. */
 	  replacement = matches[0];
 
-	  if (matches[0] && rl_completer_quote_characters && !quote_char &&
-	      rl_filename_completion_desired)
+	  should_quote = matches[0] && rl_completer_quote_characters &&
+			 rl_filename_completion_desired;
+
+#if defined (SHELL)
+	  should_quote = should_quote && (!quote_char || quote_char == '"');
+#else
+	  should_quote = should_quote && !quote_char;
+#endif
+
+	  if (should_quote)
 	    {
 	      int do_replace;
 
@@ -531,13 +620,17 @@ rl_complete_internal (what_to_do)
 		 This also checks whether the common prefix of several
 		 matches needs to be quoted.  If the common prefix should
 		 not be checked, add !matches[1] to the if clause. */
-	      if (rl_strpbrk (matches[0], rl_completer_word_break_characters))
+	      should_quote = rl_strpbrk (matches[0], rl_completer_word_break_characters) != 0;
+#if defined (SHELL)
+	      should_quote = should_quote || rl_strpbrk (matches[0], "#$`") != 0;
+#endif
+
+	      if (should_quote)
 		do_replace = matches[1] ? MULT_MATCH : SINGLE_MATCH;
 
 	      if (do_replace != NO_MATCH)
 		{
 #if defined (SHELL)
-		  /* XXX - experimental */
 		  /* Quote the replacement, since we found an
 		     embedded word break character in a potential
 		     match. */
@@ -560,8 +653,19 @@ rl_complete_internal (what_to_do)
 		    free (mtext);
 
 		  rlen = strlen (rtext);
-		  replacement = (char *)alloca (rlen + 1);
-		  strcpy (replacement, rtext);
+		  replacement = xmalloc (rlen + 1);
+		  /* If we're completing on a quoted string where the user
+		     has already supplied the opening quote, we don't want
+		     the quote in the replacement text, and we reset
+		     QUOTE_CHAR to 0 to avoid an extra closing quote. */
+		  if (quote_char == '"')
+		    {
+		      strcpy (replacement, rtext + 1);
+		      rlen--;
+		      quote_char = 0;
+		    }
+		  else
+		    strcpy (replacement, rtext);
 		  if (do_replace == MULT_MATCH)
 		    replacement[rlen - 1] = '\0';
 		  free (rtext);
@@ -569,7 +673,7 @@ rl_complete_internal (what_to_do)
 		  /* Found an embedded word break character in a potential
 		     match, so we need to prepend a quote character if we
 		     are replacing the completion string. */
-		  replacement = (char *)alloca (strlen (matches[0]) + 2);
+		  replacement = xmalloc (strlen (matches[0]) + 2);
 		  quote_char = *rl_completer_quote_characters;
 		  *replacement = quote_char;
 		  strcpy (replacement + 1, matches[0]);
@@ -579,9 +683,13 @@ rl_complete_internal (what_to_do)
 
 	  if (replacement)
 	    {
+	      rl_begin_undo_group ();
 	      rl_delete_text (start, rl_point);
 	      rl_point = start;
 	      rl_insert_text (replacement);
+	      rl_end_undo_group ();
+	      if (replacement != matches[0])
+		free (replacement);
 	    }
 
 	  /* If there are more matches, ring the bell to indicate.
@@ -591,7 +699,9 @@ rl_complete_internal (what_to_do)
 	     of the line, then add a space. */
 	  if (matches[1])
 	    {
-	      if (rl_editing_mode != vi_mode)
+	      if (what_to_do == '!')
+		goto display_matches;		/* XXX */
+	      else if (rl_editing_mode != vi_mode)
 		ding ();	/* There are other matches remaining. */
 	    }
 	  else
@@ -610,8 +720,7 @@ rl_complete_internal (what_to_do)
 		  struct stat finfo;
 		  char *filename = tilde_expand (matches[0]);
 
-		  if ((stat (filename, &finfo) == 0) &&
-		      S_ISDIR (finfo.st_mode))
+		  if ((stat (filename, &finfo) == 0) && S_ISDIR (finfo.st_mode))
 		    {
 		      if (rl_line_buffer[rl_point] != '/')
 			rl_insert_text ("/");
@@ -635,9 +744,9 @@ rl_complete_internal (what_to_do)
 	  {
 	    int i = 1;
 
+	    rl_begin_undo_group ();
 	    rl_delete_text (start, rl_point);
 	    rl_point = start;
-	    rl_begin_undo_group ();
 	    if (matches[1])
 	      {
 		while (matches[i])
@@ -657,34 +766,17 @@ rl_complete_internal (what_to_do)
 
 	case '?':
 	  {
-	    int len, count, limit, max = 0;
+	    int len, count, limit, max;
 	    int j, k, l;
 
 	    /* Handle simple case first.  What if there is only one answer? */
 	    if (!matches[1])
 	      {
-		char *temp = (char *)NULL;
-
-		if (rl_filename_completion_desired)
-		  temp = strrchr (matches[0], '/');
-
-		if (!temp)
-		  temp = matches[0];
-		else
-		  temp++;
+		char *temp;
 
+		temp = printable_part (matches[0]);
 		crlf ();
-		fprintf (rl_outstream, "%s", temp);
-#if defined (VISIBLE_STATS)
-		if (rl_filename_completion_desired && rl_visible_stats)
-		  {
-		    int extension_char;
-
-		    extension_char = stat_char (matches[0]);
-		    if (extension_char)
-		      putc (extension_char, rl_outstream);
-		  }
-#endif /* VISIBLE_STATS */
+		print_filename (temp, matches[0]);
 		crlf ();
 		goto restart;
 	      }
@@ -692,23 +784,13 @@ rl_complete_internal (what_to_do)
 	    /* There is more than one answer.  Find out how many there are,
 	       and find out what the maximum printed length of a single entry
 	       is. */
-	    for (i = 1; matches[i]; i++)
+	  display_matches:
+	    for (max = 0, i = 1; matches[i]; i++)
 	      {
 		char *temp;
 		int name_length;
 
-		/* If we are hacking filenames, then only count the characters
-		   after the last slash in the pathname. */
-		if (rl_filename_completion_desired)
-		  temp = strrchr (matches[i], '/');
-		else
-		  temp = (char *)NULL;
-
-		if (!temp)
-		  temp = matches[i];
-		else
-		  temp++;
-
+		temp = printable_part (matches[i]);
 		name_length = strlen (temp);
 
 		if (name_length > max)
@@ -766,40 +848,15 @@ rl_complete_internal (what_to_do)
 		for (j = 0, l = i; j < limit; j++)
 		  {
 		    if (l > len || !matches[l])
-		      {
-			break;
-		      }
+		      break;
 		    else
 		      {
-			char *temp = (char *)NULL;
+			char *temp;
 			int printed_length;
 
-			if (rl_filename_completion_desired)
-			  temp = strrchr (matches[l], '/');
-
-			if (!temp)
-			  temp = matches[l];
-			else
-			  temp++;
-
+			temp = printable_part (matches[l]);
 			printed_length = strlen (temp);
-			fprintf (rl_outstream, "%s", temp);
-
-#if defined (VISIBLE_STATS)
-			if (rl_filename_completion_desired &&
-			    rl_visible_stats)
-			  {
-			    int extension_char;
-
-			    extension_char = stat_char (matches[l]);
-
-			    if (extension_char)
-			      {
-				putc (extension_char, rl_outstream);
-				printed_length++;
-			      }
-			  }
-#endif /* VISIBLE_STATS */
+			printed_length += print_filename (temp, matches[l]);
 
 			if (j + 1 < limit)
 			  {
@@ -837,6 +894,7 @@ rl_complete_internal (what_to_do)
 
       free (saved_line_buffer);
     }
+  return 0;
 }
 
 #if defined (VISIBLE_STATS)
@@ -850,11 +908,18 @@ stat_char (filename)
      char *filename;
 {
   struct stat finfo;
-  int character = 0;
+  int character, r;
 
-  if (stat (filename, &finfo) == -1)
-    return (character);
+#if defined (S_ISLNK)
+  r = lstat (filename, &finfo);
+#else
+  r = stat (filename, &finfo);
+#endif
 
+  if (r == -1)
+    return (0);
+
+  character = 0;
   if (S_ISDIR (finfo.st_mode))
     character = '/';
 #if defined (S_ISLNK)
@@ -890,9 +955,9 @@ username_completion_function (text, state)
      int state;
      char *text;
 {
-#if defined (_GO32_)
+#if defined (__GO32__)
   return (char *)NULL;
-#else /* !_GO32_ */
+#else /* !__GO32__ */
   static char *username = (char *)NULL;
   static struct passwd *entry;
   static int namelen, first_char, first_char_loc;
@@ -916,7 +981,8 @@ username_completion_function (text, state)
 
   while (entry = getpwent ())
     {
-      if (strncmp (username, entry->pw_name, namelen) == 0)
+      if ((username[0] == entry->pw_name[0]) &&
+	  (strncmp (username, entry->pw_name, namelen) == 0))
 	break;
     }
 
@@ -927,7 +993,7 @@ username_completion_function (text, state)
     }
   else
     {
-      char *value = (char *)xmalloc (2 + strlen (entry->pw_name));
+      char *value = xmalloc (2 + strlen (entry->pw_name));
 
       *value = *text;
 
@@ -938,7 +1004,7 @@ username_completion_function (text, state)
 
       return (value);
     }
-#endif /* !_GO32_ */
+#endif /* !__GO32__ */
 }
 
 /* **************************************************************** */
@@ -1034,7 +1100,7 @@ completion_matches (text, entry_function)
 	      if (low > si) low = si;
 	      i++;
 	    }
-	  match_list[0] = (char *)xmalloc (low + 1);
+	  match_list[0] = xmalloc (low + 1);
 	  strncpy (match_list[0], match_list[1], low);
 	  match_list[0][low] = '\0';
 	}
@@ -1062,7 +1128,7 @@ filename_completion_function (text, state)
   static char *users_dirname = (char *)NULL;
   static int filename_len;
 
-  struct direct *entry = (struct direct *)NULL;
+  struct dirent *entry = (struct dirent *)NULL;
 
   /* If we don't have any state, then do some initialization. */
   if (!state)
@@ -1093,13 +1159,20 @@ filename_completion_function (text, state)
       users_dirname = savestring (dirname);
       {
 	char *temp_dirname;
+	int replace_dirname;
 
 	temp_dirname = tilde_expand (dirname);
 	free (dirname);
 	dirname = temp_dirname;
 
-	if (rl_symbolic_link_hook)
-	  (*rl_symbolic_link_hook) (&dirname);
+	replace_dirname = 0;
+	if (rl_directory_completion_hook)
+	  replace_dirname = (*rl_directory_completion_hook) (&dirname);
+	if (replace_dirname)
+	  {
+	    free (users_dirname);
+	    users_dirname = savestring (dirname);
+	  }
       }
       directory = opendir (dirname);
       filename_len = strlen (filename);
@@ -1127,14 +1200,12 @@ filename_completion_function (text, state)
 	}
       else
 	{
-	  /* Otherwise, if these match upto the length of filename, then
+	  /* Otherwise, if these match up to the length of filename, then
 	     it is a match. */
-	    if (((int)D_NAMLEN (entry)) >= filename_len &&
-		(entry->d_name[0] == filename[0]) &&
+	    if ((entry->d_name[0] == filename[0]) &&
+		(((int)D_NAMLEN (entry)) >= filename_len) &&
 		(strncmp (filename, entry->d_name, filename_len) == 0))
-	      {
-		break;
-	      }
+	      break;
 	}
     }
 
@@ -1145,7 +1216,6 @@ filename_completion_function (text, state)
 	  closedir (directory);
 	  directory = (DIR *)NULL;
 	}
-
       if (dirname)
 	{
 	  free (dirname);
@@ -1168,12 +1238,13 @@ filename_completion_function (text, state)
     {
       char *temp;
 
-      if (dirname && (strcmp (dirname, ".") != 0))
+      /* dirname && (strcmp (dirname, ".") != 0) */
+      if (dirname && (dirname[0] != '.' || dirname[1]))
 	{
 	  if (rl_complete_with_tilde_expansion && *users_dirname == '~')
 	    {
 	      int dirlen = strlen (dirname);
-	      temp = (char *)xmalloc (2 + dirlen + D_NAMLEN (entry));
+	      temp = xmalloc (2 + dirlen + D_NAMLEN (entry));
 	      strcpy (temp, dirname);
 	      /* Canonicalization cuts off any final slash present.  We need
 		 to add it back. */
@@ -1185,17 +1256,15 @@ filename_completion_function (text, state)
 	    }
 	  else
 	    {
-	      temp = (char *)
-		xmalloc (1 + strlen (users_dirname) + D_NAMLEN (entry));
+	      temp = xmalloc (1 + strlen (users_dirname) + D_NAMLEN (entry));
 	      strcpy (temp, users_dirname);
 	    }
 
 	  strcat (temp, entry->d_name);
 	}
       else
-	{
-	  temp = (savestring (entry->d_name));
-	}
+	temp = (savestring (entry->d_name));
+
       return (temp);
     }
 }
@@ -1241,10 +1310,11 @@ rl_tilde_expand (ignore, key)
       int len;
 
       len = end - start + 1;
-      temp = (char *)alloca (len + 1);
+      temp = xmalloc (len + 1);
       strncpy (temp, rl_line_buffer + start, len);
       temp[len] = '\0';
       homedir = tilde_expand (temp);
+      free (temp);
 
     insert:
       rl_begin_undo_group ();
