@@ -3,6 +3,24 @@
    tells which machines have what features based on the unique machine
    identifier present in Cpp. */
 
+/* Copyright (C) 1993 Free Software Foundation, Inc.
+
+   This file is part of GNU Bash, the Bourne Again SHell.
+
+   Bash is free software; you can redistribute it and/or modify it under
+   the terms of the GNU General Public License as published by the Free
+   Software Foundation; either version 2, or (at your option) any later
+   version.
+
+   Bash is distributed in the hope that it will be useful, but WITHOUT ANY
+   WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
+
+   You should have received a copy of the GNU General Public License along
+   with Bash; see the file COPYING.  If not, write to the Free Software
+   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+
 /* **************************************************************** */
 /*                                                                  */
 /*                Global Assumptions (true for most systems).       */
@@ -54,23 +72,36 @@
    in a standard library until proven otherwise. */
 #define HAVE_STRCHR
 
+/* Hardware-dependent CFLAGS. */
+#define MACHINE_CFLAGS
+
 /* **************************************************************** */
 /*								    */
 /*			Sun Microsystems Machines	      	    */
 /*								    */
 /* **************************************************************** */
 
-#if defined (sun)
-#  if defined (USGr4) || defined (__svr4__) || defined (__SVR4)
-#    define Solaris
-#  endif /* USGr4 || __svr4__ */
+/* NetBSD running on a sparc. */
+#if defined (sparc) && defined (__NetBSD__)
+#  define M_MACHINE "sun4"
+#  define M_OS "NetBSD"
+#  define SYSDEP_CFLAGS -DOPENDIR_NOT_ROBUST -DINT_GROUPS_ARRAY
+#  define SYSDEP_LDFLAGS -static
+#  define HAVE_SYS_SIGLIST
+#  define HAVE_SETLINEBUF
+#  define HAVE_GETGROUPS
+#  define HAVE_VFPRINTF
+#  define HAVE_STRERROR
+#  define VOID_SIGHANDLER
+#  define HAVE_DIRENT
+#  define HAVE_STRCASECMP
+#endif /* sparc && __NetBSD__ */
 
+#if defined (sun) && !defined (M_MACHINE)
 /* We aren't currently using GNU Malloc on Suns because of a bug in Sun's
    YP which bites us when Sun free ()'s an already free ()'ed address.
    When Sun fixes their YP, we can start using our winning malloc again. */
-#  if !defined (Solaris)
-/* #    undef USE_GNU_MALLOC */
-#  endif /* !Solaris */
+/* #undef USE_GNU_MALLOC */
 
 /* Most Sun systems have signal handler functions that are void. */
 #  define VOID_SIGHANDLER
@@ -81,6 +112,34 @@
 #  define HAVE_VFPRINTF
 #  define HAVE_GETGROUPS
 
+/* Check for SunOS4 or greater. */
+#  if defined (SunOS5)
+#    define M_OS "SunOS5"
+#    define SYSDEP_CFLAGS -DUSGr4 -DUSG -DSolaris -DOPENDIR_NOT_ROBUST \
+			  -DNO_SBRK_DECL -DINT_GROUPS_ARRAY
+#    define EXTRA_LIB_SEARCH_PATH /usr/ccs/lib
+#    if !defined (HAVE_GCC)
+#      define REQUIRED_LIBRARIES -ldl
+#      define SYSDEP_LDFLAGS -Bdynamic
+#    endif /* !HAVE_GCC */
+#    define HAVE_STRERROR
+     /* The following are only available thru the BSD compatability lib. */
+#    undef HAVE_GETWD
+#    undef HAVE_SETLINEBUF
+#  else /* !SunOS5 */
+#    if defined (SunOS4)
+#      define M_OS "SunOS4"
+#      define SYSDEP_CFLAGS -DBSD_GETPGRP -DOPENDIR_NOT_ROBUST -DTERMIOS_LDISC \
+			    -DINT_GROUPS_ARRAY
+#      define HAVE_DIRENT
+#    else /* !SunOS4 */
+#      if !defined (sparc) && !defined (__sparc__)
+#        undef VOID_SIGHANDLER
+#      endif /* !sparc */
+#      define M_OS "SunOS3"
+#    endif /* !SunOS4 */
+#  endif /* !SunOS5 */
+
 #  if defined (mc68010)
 #    define sun2
 #    define M_MACHINE "sun2"
@@ -94,35 +153,16 @@
 #    define M_MACHINE "sparc"
 #  endif
 #  if defined (i386)
-#    define Sun386i
 #    define done386
-#    define M_MACHINE "Sun386i"
+#    if !defined (SunOS5)
+#      define Sun386i
+#      define M_MACHINE "Sun386i"
+#    else
+#      define M_MACHINE "i386"
+#    endif
 #  endif /* i386 */
 
-/* Check for SunOS4 or greater. */
-#  if defined (HAVE_SHARED_LIBS)
-#    if defined (Solaris)
-#      define M_OS "SunOS5"
-#      define SYSDEP_CFLAGS -DUSGr4 -DUSG -DSolaris -DOPENDIR_NOT_ROBUST
-#      if !defined (HAVE_GCC)
-#        define REQUIRED_LIBRARIES -ldl
-#        define SYSDEP_LDFLAGS -Bdynamic
-#      endif /* !HAVE_GCC */
-       /* The following are only available thru the BSD compatability lib. */
-#      undef HAVE_GETWD
-#      undef HAVE_SETLINEBUF
-#    else /* !Solaris */
-#      define M_OS "SunOS4"
-#      define SYSDEP_CFLAGS -DBSD_GETPGRP -DOPENDIR_NOT_ROBUST
-#      define HAVE_DIRENT
-#    endif /* !Solaris */
-#  else /* !HAVE_SHARED_LIBS */
-#    if !defined (sparc) && !defined (__sparc__)
-#      undef VOID_SIGHANDLER
-#    endif /* !sparc */
-#    define M_OS "SunOS3"
-#  endif /* !HAVE_SHARED_LIBS */
-#endif /* sun */
+#endif /* sun && !M_MACHINE */
 
 /* **************************************************************** */
 /*								    */
@@ -164,7 +204,7 @@
 #  else /* !MIPSEL */
 #    define M_MACHINE "vax"
 #  endif /* !MIPSEL */
-#  define SYSDEP_CFLAGS -DBSD_GETPGRP -DTERMIOS_MISSING
+#  define SYSDEP_CFLAGS -DBSD_GETPGRP -DTERMIOS_MISSING -DTERMIOS_LDISC
 #  define M_OS "Ultrix"
 #  define HAVE_DIRENT
 #  define VOID_SIGHANDLER
@@ -189,6 +229,12 @@
 #  define USE_VFPRINTF_EMULATION
 #endif /* vax && !ultrix */
 
+/* **************************************************************** */
+/*								    */
+/*		Machines with MIPSco processors			    */
+/*								    */
+/* **************************************************************** */
+
 /* **************************************** */
 /*					    */
 /*		SGI Iris/IRIX	    	    */
@@ -197,16 +243,25 @@
 #if defined (sgi)
 #  if defined (Irix3)
 #    define M_OS "Irix3"
-#    define MIPS_CFLAGS -real_frameptr -Wf,-XNl3072
+#    if !defined (HAVE_GCC)
+#      undef MACHINE_CFLAGS
+#      define MACHINE_CFLAGS -real_frameptr -Wf,-XNl3072
+#    endif
 #    undef HAVE_ALLOCA
 #  endif /* Irix3 */
 #  if defined (Irix4)
 #    define M_OS "Irix4"
-#    define MIPS_CFLAGS -Wf,-XNl3072
+#    if !defined (HAVE_GCC)
+#      undef MACHINE_CFLAGS
+#      define MACHINE_CFLAGS -Wf,-XNl3072
+#    endif
 #  endif /* Irix4 */
 #  if defined (Irix5)
 #    define M_OS "Irix5"
-#    define MIPS_CFLAGS -Wf,-XNl3072
+#    if !defined (HAVE_GCC)
+#      undef MACHINE_CFLAGS
+#      define MACHINE_CFLAGS -Wf,-XNl3072
+#    endif
 #  endif /* Irix5 */
 #  define M_MACHINE "sgi"
 #  define HAVE_GETGROUPS
@@ -215,12 +270,20 @@
 #  define HAVE_SETLINEBUF
 #  define HAVE_VFPRINTF
 #  define REQUIRED_LIBRARIES -lsun
-#  if defined (HAVE_GCC) || !defined (mips)
-#    undef MIPS_CFLAGS
-#    define MIPS_CFLAGS
-#  endif /* HAVE_GCC || !mips */
-#  define SGI_CFLAGS -DUSG -DPGRP_PIPE
-#  define SYSDEP_CFLAGS SGI_CFLAGS MIPS_CFLAGS
+   /* SGI cc uses ansi c features *without* defining __STDC__ */
+#  if defined (__EXTENSIONS__) && !defined (__STDC__)
+#    define ANSIC -D__STDC__
+#  else
+#    define ANSIC
+#  endif /* !__EXTENSIONS__ || __STDC__ */
+#  if defined (Irix5)
+#    define SGI_CFLAGS -DUSG -DPGRP_PIPE -DHAVE_BCOPY -DHAVE_GETPW_DECLS \
+		       -DHAVE_SOCKETS -DNO_SBRK_DECL
+#  else
+#    define SGI_CFLAGS -DUSG -DPGRP_PIPE -DHAVE_BCOPY -DHAVE_GETPW_DECLS \
+		       -DHAVE_SOCKETS
+#  endif /* !Irix5 */
+#  define SYSDEP_CFLAGS SGI_CFLAGS MACHINE_CFLAGS ANSIC
 #endif  /* sgi */
 
 /* ************************ */
@@ -231,7 +294,7 @@
 #if defined (nec_ews)
 #  if defined (SYSTYPE_SYSV) || defined (USGr4)
 #    define M_MACHINE "ews4800"
-#    define M_OS USG
+#    define M_OS "USG"
 #    define HAVE_VFPRINTF
 #    define VOID_SIGHANDLER
 #    define HAVE_STRERROR
@@ -245,27 +308,60 @@
 #    endif /* !HAVE_GCC */
 #    if defined (MIPSEB)
 #      if !defined (HAVE_GCC)
-#        define SYSDEP_CFLAGS -Wf,-XNl3072 -DUSGr4 -DUSGr3 -D_POSIX_JOB_CONTROL
-#      else
-#        define SYSDEP_CFLAGS -DUSGr4 -DUSGr3 -D_POSIX_JOB_CONTROL
-#      endif /* HAVE_GCC */
+#        undef MACHINE_CFLAGS
+#        define MACHINE_CFLAGS -Wf,-XNl3072
+#      endif
+#      define SYSDEP_CFLAGS MACHINE_CFLAGS -DUSGr4 -DUSGr3 -D_POSIX_JOB_CONTROL
 #    else /* !MIPSEB */
 #      define SYSDEP_CFLAGS -DUSGr4
 #    endif /* MIPSEB */
 #  else /* !SYSTYPE_SYSV && !USGr4 */
-#    define M_OS Bsd
+#    define M_OS "Bsd"
 #  endif /* !SYSTYPE_SYSV && !USGr4 */
 #  define HAVE_SYS_SIGLIST
 #  define HAVE_SETLINEBUF
 #  define HAVE_GETGROUPS
 #endif /* nec_ews */
 
+/* ************************ */
+/*			    */
+/*  Generic MIPS SVR4, 4.2  */
+/*			    */
+/* ************************ */
+#if defined (MIPSEB) && defined (USGr4)
+#  define M_MACHINE "MIPSEB"
+#  define M_OS "USG"
+#  if defined (sony) && !defined (HAVE_GCC)
+#    undef MACHINE_CFLAGS
+#    define MACHINE_CFLAGS -Wf,-XNl3072
+#  endif
+/* XXX - os/svr4.h -- for the future -- XXX */
+#  undef HAVE_GETWD
+#  define HAVE_DIRENT
+#  define HAVE_STRERROR
+#  define HAVE_VFPRINTF
+#  define VOID_SIGHANDLER
+#  define HAVE_SYS_SIGLIST
+#  define HAVE_SETLINEBUF
+#  define HAVE_GETGROUPS
+/* alloca */
+#  if !defined (HAVE_GCC)
+#    define EXTRA_LIB_SEARCH_PATH /usr/ucblib
+#    define REQUIRED_LIBRARIES -lc -lucb
+#  endif /* !HAVE_GCC */
+#  if defined (USGr4_2)
+#    define SYSDEP_CFLAGS MACHINE_CFLAGS -DUSGr4 -DUSGr4_2
+#  else
+#    define SYSDEP_CFLAGS MACHINE_CFLAGS -DUSGr4
+#  endif /* !USGr4_2 */
+#endif
+
 /* ************************ */
 /*			    */
 /*	    Sony	    */
 /*			    */
 /* ************************ */
-#if defined (sony)
+#if defined (sony) && !defined (M_MACHINE)
 #  if defined (MIPSEB)
 #    define M_MACHINE "MIPSEB"
 #  else /* !MIPSEB */
@@ -286,10 +382,10 @@
 #    endif /* !HAVE_GCC */
 #    if defined (MIPSEB)
 #      if !defined (HAVE_GCC)
-#        define SYSDEP_CFLAGS -Wf,-XNl3072 -DUSGr4
-#      else
-#        define SYSDEP_CFLAGS -DUSGr4
-#      endif /* HAVE_GCC */
+#        undef MACHINE_CFLAGS
+#        define MACHINE_CFLAGS -Wf,-XNl3072
+#      endif
+#      define SYSDEP_CFLAGS MACHINE_CFLAGS -DUSGr4
 #    else /* !MIPSEB */
 #      define SYSDEP_CFLAGS -DUSGr4
 #    endif /* !MIPSEB */
@@ -302,6 +398,44 @@
 #  define HAVE_GETGROUPS
 #endif /* sony */
 
+/* ******************************* */
+/*			           */
+/*  Ardent Titan OS v2.2 and later */
+/*			           */
+/* ******************************* */
+#if defined (ardent)
+#  define M_MACHINE "Ardent Titan"
+#  define M_OS "Bsd"
+#  if defined (titan)
+#    undef HAVE_GETGROUPS
+#  else
+#    define HAVE_GETGROUPS
+#  endif /* !titan */
+#  define HAVE_SYS_SIGLIST
+#  define HAVE_SETLINEBUF
+#  define SYSDEP_CFLAGS -43 -w
+#  define SYSDEP_LDFLAGS -43
+#  undef HAVE_ALLOCA
+#  undef USE_GNU_MALLOC
+#  undef HAVE_VFPRINTF
+#  undef HAVE_DIRENT_H
+#endif /* ardent */
+
+/* ************************ */
+/*			    */
+/*	  Stardent	    */
+/*			    */
+/* ************************ */
+#if defined (stardent) && !defined (M_MACHINE)
+#  define M_MACHINE "Stardent"
+#  define M_OS "USG"
+#  define HAVE_SYS_SIGLIST
+#  define USE_TERMCAP_EMULATION
+#  define VOID_SIGHANDLER
+#  undef HAVE_GETWD
+#  undef HAVE_ALLOCA
+#endif /* stardent */
+
 /* ******************************** */
 /*				    */
 /*	   MIPS RISC/os		    */
@@ -330,16 +464,15 @@
    /* Special things for machines from MIPS Co. */
 #  define MIPS_CFLAGS -DOPENDIR_NOT_ROBUST -DPGRP_PIPE
 
-#  if defined (HAVE_GCC)
-#    define SYSDEP_CFLAGS MIPS_CFLAGS
-#  else /* !HAVE_GCC */
-#    define SYSDEP_CFLAGS -Wf,-XNl3072 -systype bsd43 MIPS_CFLAGS
+#  if !defined (HAVE_GCC)
+#    undef MACHINE_CFLAGS
+#    define MACHINE_CFLAGS -Wf,-XNl3072 -systype bsd43
 #  endif /* !HAVE_GCC */
+#  define SYSDEP_CFLAGS MACHINE_CFLAGS MIPS_CFLAGS
 #  define HAVE_SYS_SIGLIST
 #  define HAVE_SETLINEBUF
 #  define HAVE_VFPRINTF
 #  define HAVE_GETGROUPS
-   /* This is actually present but unavailable in the BSD universe? */
 #  undef HAVE_UNISTD_H
 #  if !defined (HAVE_RESOURCE)
 #    define HAVE_RESOURCE
@@ -361,7 +494,6 @@
 #  endif /* HAVE_GCC */
 #  define HAVE_SYS_SIGLIST
 #  define HAVE_SETLINEBUF
-   /* #define HAVE_VFPRINTF */
 #  define HAVE_GETGROUPS
 #endif /* pyr */
 
@@ -397,10 +529,13 @@
 #    define done386
 #    define M_MACHINE "Symmetry"
 #    define M_OS "Dynix"
-#    define SYSDEP_CFLAGS -DUSG -DUSGr3 -DHAVE_GETDTABLESIZE -DHAVE_SETDTABLESIZE
+#    define DYNIX_CFLAGS -DHAVE_GETDTABLESIZE -DHAVE_SETDTABLESIZE \
+			 -DHAVE_GETPW_DECLS -DHAVE_SOCKETS
+#    define SYSDEP_CFLAGS -DUSG -DUSGr3 DYNIX_CFLAGS
 #    define HAVE_DIRENT
 #    define HAVE_VFPRINTF
 #    define VOID_SIGHANDLER
+/* Might need to add -lsocket -linet -lnsl to the list of libraries. */
 #    define REQUIRED_LIBRARIES -lPW -lseq
 #    undef HAVE_GETWD
 #    undef HAVE_RESOURCE
@@ -423,17 +558,22 @@
 /* NeXT 3.x on i386 */
 #  if !defined (done386) && defined (NeXT)
 #    define done386
-#    define M_MACHINE "NeXT"
-#    define M_OS Bsd
+#    define M_MACHINE "i386"
+#    define M_OS "NeXTstep"
 #    define HAVE_VFPRINTF
 #    define HAVE_SYS_SIGLIST
 #    define HAVE_GETGROUPS
 #    define HAVE_STRERROR
 #    define VOID_SIGHANDLER
+#    if !defined (HAVE_RESOURCE)
+#      define HAVE_RESOURCE
+#    endif
+#    define HAVE_STRCASECMP
 #    undef HAVE_GETWD
 #    undef HAVE_GETCWD
 #    undef USE_GNU_MALLOC
-#    define SYSDEP_CFLAGS -DHAVE_RESOURCE -DMKFIFO_MISSING
+#    undef HAVE_DIRENT_H
+#    define SYSDEP_CFLAGS -DNeXT -DMKFIFO_MISSING -DRLOGIN_PGRP_BUG
 #  endif
 
 /* Generic 386 clone running Mach (4.3 BSD-compatible). */
@@ -454,9 +594,10 @@
 #    define HAVE_VFPRINTF
 #    define VOID_SIGHANDLER
 #    if defined (AIX_13)	/* AIX PS/2 1.3 */
+#      define SYSDEP_CFLAGS -DTERMIOS_LDISC
 #      define REQUIRED_LIBRARIES -lc_s
 #    else
-#      define SYSDEP_CFLAGS -D_BSD
+#      define SYSDEP_CFLAGS -D_BSD -DTERMIOS_LDISC
 #      define REQUIRED_LIBRARIES -lbsd -lc_s
 #    endif /* !AIX_13 */
 #    define HAVE_GETGROUPS
@@ -502,12 +643,13 @@
 #    define USE_TERMCAP_EMULATION
 #    if defined (HAVE_GCC)
 #      define SYSDEP_LDFLAGS -posix
+#      define ISC_POSIX
 #    else
 #      define REQUIRED_LIBRARIES -lPW
 #      define SYSDEP_LDFLAGS -Xp
 #      define ISC_POSIX -Xp
 #    endif
-#    define ISC_SYSDEPS -DUSGr3 -DPGRP_PIPE -DHAVE_GETPW_DECLS -D_POSIX_SOURCE -DOPENDIR_NOT_ROBUST
+#    define ISC_SYSDEPS -DUSGr3 -DPGRP_PIPE -DHAVE_GETPW_DECLS -D_POSIX_SOURCE -DOPENDIR_NOT_ROBUST -DMEMMOVE_MISSING
 #    if defined (__STDC__)
 #      if defined (HAVE_GCC)
 #        define ISC_EXTRA -DO_NDELAY=O_NONBLOCK
@@ -527,7 +669,7 @@
 #    define done386
 #    define M_MACHINE "i386"
 #    define M_OS "Xenix"
-#    define XENIX_CFLAGS -DUSG -DUSGr3
+#    define XENIX_CFLAGS -DUSG -DUSGr3 -DMEMMOVE_MISSING
 
 #    if defined (XENIX_22)
 #      define XENIX_EXTRA -DREVERSED_SETVBUF_ARGS
@@ -535,7 +677,7 @@
 #    else /* !XENIX_22 */
 #      define HAVE_DIRENT
 #      if defined (XENIX_23)
-#        define XENIX_EXTRA
+#        define XENIX_EXTRA -DLD_HAS_NO_DASH_L
 #        define REQUIRED_LIBRARIES -ldir
 #      else /* !XENIX_23 */
 #        define XENIX_EXTRA -xenix
@@ -551,6 +693,7 @@
 #    define ALLOCA_OBJ x386-alloca.o
 #    undef HAVE_ALLOCA
 #    undef HAVE_GETWD
+#    undef HAVE_RESOURCE
 #  endif /* Xenix386 */
 
 /* SCO UNIX 3.2 chip@count.tct.com (Chip Salzenberg) */
@@ -578,11 +721,29 @@
 #    endif /* !HAVE_GCC */
 #  endif /* SCO Unix on 386 boxes. */
 
+#  if !defined (done386) && defined (__OSF1__)
+#    define done386
+#    define M_MACHINE "i386"
+#    define M_OS "OSF1"
+#    define HAVE_SYS_SIGLIST
+#    define HAVE_SETLINEBUF
+#    define HAVE_VFPRINTF
+#    define HAVE_STRERROR
+#    define HAVE_GETGROUPS
+#    define VOID_SIGHANDLER
+#    define HAVE_BCOPY
+#    define USE_TERMCAP_EMULATION
+#    define SYSDEP_CFLAGS -D_BSD
+#    define SYSDEP_LDFLAGS
+#    define REQUIRED_LIBRARIES -lbsd
+#  endif /* OSF/1 */
+
 /* BSDI BSD/386 running on a 386 or 486. */
-#  if !defined (done386) && defined (bsdi)
+#  if !defined (done386) && defined (__bsdi__)
 #    define done386
 #    define M_MACHINE "i386"
 #    define M_OS "BSD386"
+#    define SYSDEP_CFLAGS -DOPENDIR_NOT_ROBUST -DINT_GROUPS_ARRAY
 #    define HAVE_SYS_SIGLIST
 #    define HAVE_SETLINEBUF
 #    define HAVE_GETGROUPS
@@ -590,13 +751,47 @@
 #    define HAVE_STRERROR
 #    define VOID_SIGHANDLER
 #    define HAVE_DIRENT
+#    define HAVE_STRCASECMP
 #  endif /* !done386 && bsdi */
 
+/* NetBSD running on a 386 or 486. */
+#  if !defined (done386) && defined (__NetBSD__)
+#    define done386
+#    define M_MACHINE "i386"
+#    define M_OS "NetBSD"
+#    define SYSDEP_CFLAGS -DOPENDIR_NOT_ROBUST -DINT_GROUPS_ARRAY
+#    define HAVE_SYS_SIGLIST
+#    define HAVE_SETLINEBUF
+#    define HAVE_GETGROUPS
+#    define HAVE_VFPRINTF
+#    define HAVE_STRERROR
+#    define VOID_SIGHANDLER
+#    define HAVE_DIRENT
+#    define HAVE_STRCASECMP
+#  endif /* !done386 && __NetBSD__ */
+
+/* FreeBSD running on a 386 or 486. */
+#  if !defined (done386) && defined (__FreeBSD__)
+#    define done386
+#    define M_MACHINE "i386"
+#    define M_OS "FreeBSD"
+#    define SYSDEP_CFLAGS -DOPENDIR_NOT_ROBUST -DINT_GROUPS_ARRAY
+#    define HAVE_SYS_SIGLIST
+#    define HAVE_SETLINEBUF
+#    define HAVE_GETGROUPS
+#    define HAVE_VFPRINTF
+#    define HAVE_STRERROR
+#    define VOID_SIGHANDLER
+#    define HAVE_DIRENT
+#    define HAVE_STRCASECMP
+#  endif /* !done386 && __FreeBSD__ */
+
 /* Jolitz 386BSD running on a 386 or 486. */
 #  if !defined (done386) && defined (__386BSD__)
 #    define done386
 #    define M_MACHINE "i386"
 #    define M_OS "_386BSD"
+#    define SYSDEP_CFLAGS -DOPENDIR_NOT_ROBUST -DINT_GROUPS_ARRAY
 #    define HAVE_SYS_SIGLIST
 #    define HAVE_SETLINEBUF
 #    define HAVE_GETGROUPS
@@ -604,13 +799,15 @@
 #    define HAVE_STRERROR
 #    define VOID_SIGHANDLER
 #    define HAVE_DIRENT
+#    define HAVE_STRCASECMP
 #  endif /* !done386 && __386BSD__ */
 
 #  if !defined (done386) && (defined (__linux__) || defined (linux))
 #    define done386
 #    define M_MACHINE "i386"
-#    define M_OS Linux
-#    define SYSDEP_CFLAGS -DUSG -DUSGr3 -DHAVE_GETDTABLESIZE -DHAVE_BCOPY
+#    define M_OS "Linux"
+#    define SYSDEP_CFLAGS -DUSG -DUSGr3 -DHAVE_GETDTABLESIZE -DHAVE_BCOPY \
+			  -DHAVE_GETPW_DECLS -DHAVE_GETHOSTNAME
 #    define REQUIRED_LIBRARIES
 #    define HAVE_GETGROUPS
 #    define HAVE_STRERROR
@@ -627,6 +824,45 @@
 #    undef HAVE_GETWD
 #  endif  /* !done386 && __linux__ */
 
+/* QNX 4.2 with GCC pt@flard.ocunix.on.ca (Paul Trunley) */
+#  if !defined (done386) && defined (qnx)
+#    define done386
+#    define M_MACHINE "i386"
+#    define M_OS "QNX"
+#    define SYSDEP_CFLAGS -D_POSIX_SOURCE -O2 -DUSG
+#    define HAVE_VFPRINTF
+#    define VOID_SIGHANDLER
+#    define HAVE_GCC
+#    define HAVE_FIXED_INCLUDES
+#    define HAVE_STRERROR
+#    define HAVE_GETGROUPS
+#    undef USE_GNU_MALLOC
+#  endif /* QNX 4.2 with GCC */
+
+/* Lynx 2.1.0 (Mike Brennen <mbrennen@maverick.intecom.com>) */
+#  if !defined (done386) && (defined (__Lynx__) || defined (Lynx))
+#    define done386
+#    define M_MACHINE "i386"
+#    define M_OS "Lynx"
+#    define SYSDEP_CFLAGS -DHAVE_GETDTABLESIZE -DHAVE_BCOPY
+#    define REQUIRED_LIBRARIES -lc_p
+#    define HAVE_GETGROUPS
+#    define VOID_SIGHANDLER
+#    define HAVE_SYS_SIGLIST
+#    define HAVE_VFPRINTF
+#    define HAVE_VARARGS_H
+#    if defined (__GNUC__)
+#      define HAVE_FIXED_INCLUDES
+#    endif /* __GNUC__ */
+/* Actually, Lynx does have unistd.h, but it defines _POSIX_VERSION,
+   and doesn't supply a fully compatible job control package.  We just
+   pretend that it doesn't have it. */
+#  undef HAVE_UNISTD_H
+/* Lynx's wait structure reverses w_Stopval and w_Stopsig - don't use it */
+#  undef HAVE_WAIT_H
+#  undef HAVE_DIRENT_H
+#  endif  /* !done386 && __Lynx__ */
+
 /* Assume a generic 386 running Sys V Release 3. */
 #  if !defined (done386)
 #    define done386
@@ -663,7 +899,8 @@
 #  define HAVE_STRERROR
 #  define USE_GNU_MALLOC
 #  define LD_HAS_NO_DASH_L
-#  define SYSDEP_CFLAGS -DTERMIOS_MISSING -DMKFIFO_MISSING -DBSD_GETPGRP -w
+#  define SYSDEP_CFLAGS -DTERMIOS_MISSING -DMKFIFO_MISSING \
+	-DBSD_GETPGRP -DRLOGIN_PGRP_BUG -w
    /* Actually, Alliant does have unistd.h, but it defines _POSIX_VERSION,
       and doesn't supply a fully compatible job control package.  We just
       pretend that it doesn't have it. */
@@ -711,15 +948,20 @@
 /* ************************ */
 #if defined (NeXT) && !defined (M_MACHINE)
 #  define M_MACHINE "NeXT"
-#  define M_OS "Bsd"
+#  define M_OS "NeXTstep"
 #  define HAVE_VFPRINTF
 #  define HAVE_SYS_SIGLIST
 #  define HAVE_GETGROUPS
 #  define HAVE_STRERROR
 #  define VOID_SIGHANDLER
+#  if !defined (HAVE_RESOURCE)
+#    define HAVE_RESOURCE
+#  endif
+#  define HAVE_STRCASECMP
 #  undef HAVE_GETWD
 #  undef HAVE_GETCWD
-#  define SYSDEP_CFLAGS -DHAVE_RESOURCE -DMKFIFO_MISSING
+#  undef HAVE_DIRENT_H
+#  define SYSDEP_CFLAGS -DMKFIFO_MISSING -DRLOGIN_PGRP_BUG
 #  undef USE_GNU_MALLOC
 #endif /* NeXT */
 
@@ -737,6 +979,8 @@
 #  define HAVE_STRERROR
 #  define HAVE_VFPRINTF
 #  define VOID_SIGHANDLER
+#  define HAVE_STRCASECMP
+#  define SYSDEP_CFLAGS -DHAVE_GETDTABLESIZE -DHAVE_BCOPY -DHAVE_RESOURCE
 #  undef HAVE_ALLOCA
 #endif /* hp9000 && __BSD_4_4__ */
 
@@ -790,14 +1034,14 @@
 /* This is for HP-UX systems earlier than HP-UX 6.2 -- no job control. */
 #  if defined (HPUX_USG)
 #    define M_OS "USG"
-#    define HPUX_SYSDEP_CFLAGS -Dhpux
+#    define HPUX_CFLAGS -Dhpux
 #    define REQUIRED_LIBRARIES -lPW -lBSD
 #    undef HAVE_WAIT_H
 #    define HPUX_EXTRA
 #  else /* !HPUX_USG */
 
 /* All of the other operating systems need HPUX to be defined. */
-#    define HPUX_EXTRA -DHPUX
+#    define HPUX_EXTRA -DHPUX -DHAVE_GETHOSTNAME -DUSG
 
      /* HPUX 6.2 .. 6.5 require -lBSD for getwd (), and -lPW for alloca (). */
 #    if defined (HPUX_6)
@@ -811,6 +1055,7 @@
 #    if defined (HPUX_7)
 #      define M_OS "hpux_7"
 #      define REQUIRED_LIBRARIES -lPW
+#      define HPUX_CFLAGS -DHAVE_SOCKETS
 #      undef HAVE_GETWD
 #      undef USE_GNU_MALLOC
 #    endif /* HPUX_7 */
@@ -823,16 +1068,20 @@
 #      undef HAVE_ALLOCA
 #      undef HAVE_GETWD
 #      undef USE_GNU_MALLOC
+#      define HPUX_CFLAGS -DNO_SBRK_DECL -DHAVE_SOCKETS
 #    endif /* HPUX_8 */
 
      /* HP-UX 9.0 reportedly fixes the alloca problems present in the 8.0
         release.  If so, -lPW is required to include it. */
 #    if defined (HPUX_9)
 #      define M_OS "hpux_9"
-#      define REQUIRED_LIBRARIES -lPW
+#      if !defined (__GNUC__)
+#        define REQUIRED_LIBRARIES -lPW
+#        undef USE_GNU_MALLOC
+#      endif
 #      undef HAVE_RESOURCE
 #      undef HAVE_GETWD
-#      undef USE_GNU_MALLOC
+#      define HPUX_CFLAGS -DNO_SBRK_DECL -DHAVE_SOCKETS -DHAVE_GETHOSTNAME
 #    endif /* HPUX_9 */
 
 #  endif /* !HPUX_USG */
@@ -846,14 +1095,34 @@
 #  define USE_TERMCAP_EMULATION
 #  define SEARCH_LIB_NEEDS_SPACE
 
-#  if defined (HPUX_SYSDEP_CFLAGS)
-#    define SYSDEP_CFLAGS HPUX_SYSDEP_CFLAGS HPUX_EXTRA
-#  else /* !HPUX_SYSDEP_CFLAGS */
+#  if defined (HPUX_CFLAGS)
+#    define SYSDEP_CFLAGS HPUX_CFLAGS HPUX_EXTRA
+#  else /* !HPUX_CFLAGS */
 #    define SYSDEP_CFLAGS HPUX_EXTRA
-#  endif /* !HPUX_SYSDEP_CFLAGS */
+#  endif /* !HPUX_CFLAGS */
 
 #endif /* hpux */
 
+/* ************************ */
+/*                          */
+/*        MIPS OSF/1        */
+/*                          */
+/* ************************ */
+#  if defined (MIPSEL) && defined (__OSF1__)
+#    define M_MACHINE "mips"
+#    define M_OS "OSF1"
+#    define HAVE_SYS_SIGLIST
+#    define HAVE_SETLINEBUF
+#    define HAVE_VFPRINTF
+#    define HAVE_STRERROR
+#    define HAVE_GETGROUPS
+#    define VOID_SIGHANDLER
+#    define HAVE_BCOPY
+#    define USE_TERMCAP_EMULATION
+#    define SYSDEP_CFLAGS -D_BSD
+#    define REQUIRED_LIBRARIES -lbsd
+#  endif /* MIPSEL && __OSF1__ */
+
 /* ************************ */
 /*                          */
 /*        HP OSF/1          */
@@ -907,6 +1176,24 @@
 #  define HAVE_SYS_SIGLIST
 #endif /* __i860 && __PARAGON__ */
 
+/* ************************ */
+/*                          */
+/* IBM AIX/ESA (OSF/1)      */
+/*                          */
+/* ************************ */
+#if defined(AIXESA) || (defined(__ibmesa) && defined(_AIX))
+#  define M_MACHINE "IBMESA"
+#  define M_OS "OSF1"
+#  define HAVE_GETGROUPS
+#  define HAVE_SETLINEBUF
+#  define HAVE_VPRINTF
+#  define VOID_SIGHANDLER
+#  define HAVE_STRERROR
+#  define HAVE_SYS_SIGLIST
+#  define HAVE_ALLOCA_H		/* hack for AIX/ESA, which has malloc.h */
+#  undef USE_GNU_MALLOC
+#endif /* AIXESA || (__ibmesa && _AIX) */
+
 /* ************************ */
 /*			    */
 /*	    Xenix286	    */
@@ -916,7 +1203,7 @@
 #  define M_MACHINE "i286"
 #  define M_OS "Xenix"
 
-#  define XENIX_CFLAGS -DUSG -DUSGr3
+#  define XENIX_CFLAGS -DUSG -DUSGr3 -DMEMMOVE_MISSING
 
 #  if defined (XENIX_22)
 #    define XENIX_EXTRA -DREVERSED_SETVBUF_ARGS
@@ -924,7 +1211,7 @@
 #  else /* !XENIX_22 */
 #    define HAVE_DIRENT
 #    if defined (XENIX_23)
-#      define XENIX_EXTRA
+#      define XENIX_EXTRA -DLD_HAS_NO_DASH_L
 #      define REQUIRED_LIBRARIES -ldir
 #    else /* !XENIX_23 */
 #      define XENIX_EXTRA -xenix
@@ -936,6 +1223,7 @@
 #  define SYSDEP_CFLAGS XENIX_CFLAGS XENIX_EXTRA
 #  undef HAVE_ALLOCA
 #  undef HAVE_GETWD
+#  undef HAVE_RESOURCE
 #endif /* Xenix286 */
 
 /* ************************ */
@@ -967,10 +1255,10 @@
 #  define HAVE_GETGROUPS
 #  define USE_TERMCAP_EMULATION
 #  if !defined (HAVE_GCC)
-#    define SYSDEP_CFLAGS -a -DNLS -DUSGr3 -DHAVE_BCOPY
-#  else /* HAVE_GCC */
-#    define SYSDEP_CFLAGS -DNLS -DUSGr3 -DHAVE_BCOPY
-#  endif /* HAVE_GCC */
+#    undef MACHINE_CFLAGS
+#    define MACHINE_CFLAGS -a
+#  endif /* !HAVE_GCC */
+#  define SYSDEP_CFLAGS MACHINE_CFLAGS -DNLS -DUSGr3 -DHAVE_BCOPY
 #  undef USE_GNU_MALLOC
 #  undef HAVE_ALLOCA
 #  undef HAVE_RESOURCE
@@ -1029,7 +1317,7 @@
 #    endif /* !HAVE_GCC */
 #    define HAVE_GETGROUPS
 #    define HAVE_RESOURCE
-#    define SYSDEP_CFLAGS -DUSGr4
+#    define SYSDEP_CFLAGS -DUSGr4 -DNO_SBRK_DECL
 #    endif /* USGr4 */
 #endif /* u370 */
 
@@ -1174,6 +1462,28 @@
 #  undef USE_GNU_MALLOC
 #endif /* drs6000 */
 
+/* ******************************************** */
+/*						*/
+/*   System V Release 4 on the Sparc (generic)  */
+/*						*/
+/* ******************************************** */
+#if defined (sparc) && defined (__svr4__) && !defined (M_MACHINE)
+#  define M_MACHINE "sparc"
+#  define M_OS "SVR4"
+#  define SYSDEP_CFLAGS -DUSG -DUSGr4 -DHAVE_UID_T
+#  define HAVE_DIRENT
+#  define HAVE_VFPRINTF
+#  define HAVE_GETGROUPS
+#  define HAVE_STRERROR
+#  define VOID_SIGHANDLER
+#  define USE_GNU_TERMCAP
+#  if !defined (__GNUC__)
+#    undef HAVE_ALLOCA
+#  endif
+#  undef HAVE_BCOPY
+#  undef USE_GNU_MALLOC
+#endif /* sparc && __svr4__ */
+
 /* ******************************************** */
 /*						*/
 /*  System V Release 4 on the Commodore Amiga   */
@@ -1282,18 +1592,19 @@ MAKE = make
 /*			    */
 /* ************************ */
 /* This is for the DG AViiON box (runs DG/UX with both AT&T & BSD features.) */
+/* DG/UX comes standard with Gcc. */
 #if defined (__DGUX__) || defined (DGUX)
 #  define M_OS "USG"
 #  if !defined (_M88KBCS_TARGET)
 #    define M_MACHINE "AViiON"
-#    define SYSDEP_CFLAGS -D_DGUX_SOURCE -DPGRP_PIPE
 #    define REQUIRED_LIBRARIES -ldgc
 #  else /* _M88KBCS_TARGET */
 #    define M_MACHINE "m88kBCS_AV"
-#    define SYSDEP_CFLAGS -D_DGUX_SOURCE -D_M88K_SOURCE -DPGRP_PIPE
+#    undef MACHINE_CFLAGS
+#    define MACHINE_CFLAGS -D_M88K_SOURCE
 #    undef HAVE_RESOURCE
 #  endif /* _M88KBCS_TARGET */
-   /* DG/UX comes standard with Gcc. */
+#  define SYSDEP_CFLAGS MACHINE_CFLAGS -D_DGUX_SOURCE -DPGRP_PIPE
 #  define HAVE_GCC
 #  define HAVE_FIXED_INCLUDES
 #  define HAVE_STRERROR
@@ -1398,7 +1709,9 @@ MAKE = make
 #  define VOID_SIGHANDLER
 #  define HAVE_GETGROUPS
 #  undef HAVE_GETWD
-#  undef HAVE_GETCWD
+#  if !defined (USGr4)
+#    undef HAVE_GETCWD
+#  endif
 #  undef HAVE_ALLOCA
 #endif /* m88k && M88100 */
 
@@ -1428,63 +1741,40 @@ MAKE = make
 #  if !defined (HAVE_GCC)
 #    define REQUIRED_LIBRARIES -lPW
      /* Disable stack/frame-pointer optimization, incompatible with alloca */
-#    define SYSDEP_CFLAGS -DUSGr3 -W2,-aat
-#  else /* HAVE_GCC */
-#    define SYSDEP_CFLAGS -DUSGr3
-#  endif /* HAVE_GCC */
+#    undef MACHINE_CFLAGS
+#    define MACHINE_CFLAGS -W2,-aat
+#  endif /* !HAVE_GCC */
+#  define SYSDEP_CFLAGS -DUSGr3 MACHINE_CFLAGS
 #  define HAVE_VFPRINTF
 #  define USE_TERMCAP_EMULATION
 #  define VOID_SIGHANDLER
 #  undef HAVE_GETWD
 #endif /* tower32 */
 
-/* ************************ */
-/*			    */
-/*    Ardent Titan OS v2.2  */
-/*			    */
-/* ************************ */
-#if defined (ardent)
-#  define M_MACHINE "Ardent Titan"
-#  define M_OS "Bsd"
-#  if !defined (titan)
-#    define HAVE_GETGROUPS
-#  endif /* !titan */
-#  define HAVE_SYS_SIGLIST
-#  define HAVE_SETLINEBUF
-#  define SYSDEP_CFLAGS -43 -w
-#  define SYSDEP_LDFLAGS -43
-#  undef HAVE_ALLOCA
-#  undef USE_GNU_MALLOC
-#  undef HAVE_VFPRINTF
-#endif /* ardent */
-
-/* ************************ */
-/*			    */
-/*	  Stardent	    */
-/*			    */
-/* ************************ */
-#if defined (stardent) && !defined (M_MACHINE)
-#  define M_MACHINE "Stardent"
-#  define M_OS "USG"
-#  define HAVE_SYS_SIGLIST
-#  define USE_TERMCAP_EMULATION
-#  define VOID_SIGHANDLER
-#  undef HAVE_GETWD
-#  undef HAVE_ALLOCA
-#endif /* stardent */
-
 /* ************************ */
 /*			    */
 /*	Concurrent	    */
 /*			    */
 /* ************************ */
 #if defined (concurrent)
-   /* Use the BSD universe (`universe ucb') */
 #  define M_MACHINE "Concurrent"
-#  define M_OS "Bsd"
-#  define HAVE_SYS_SIGLIST
-#  define HAVE_SETLINEBUF
-#  define HAVE_GETGROUPS
+#  if defined (USE_BSD_UNIVERSE)
+     /* Use the BSD universe (`universe ucb') */
+#    define M_OS "Bsd"
+#    define HAVE_SYS_SIGLIST
+#    define HAVE_SETLINEBUF
+#    define HAVE_GETGROUPS
+#  else /* !USE_BSD_UNIVERSE */
+   /* Concurrent 7000 with RTU 6.1A using the ATT universe (`universe att') */
+#    define M_OS "USG"
+#    define SYSDEP_CFLAGS -DHAVE_BCOPY -DHAVE_UID_T -DHAVE_GETDTABLESIZE -Dmc7000
+#    define REQUIRED_LIBRARIES -ljobs
+#    define HAVE_VPRINTF
+#    define HAVE_GETGROUPS
+#    define HAVE_DUP2
+#    define HAVE_DIRENT
+#    define HAVE_SYS_SIGLIST
+#  endif /* !USE_BSD_UNIVERSE */
 #endif /* concurrent */
 
 /* **************************************************************** */
@@ -1521,7 +1811,11 @@ MAKE = make
 #  endif
 #  if defined (CrayYMP) && !defined (M_MACHINE)
 #    define M_MACHINE "CrayYMP"
-#    define CRAY_STACK -DCRAY_STACKSEG_END=getb67
+#    if RELEASE_LEVEL >= 7000
+#      define CRAY_STACK -DCRAY_STACKSEG_END=GETB67
+#    else
+#      define CRAY_STACK -DCRAY_STACKSEG_END=getb67
+#    endif /* RELEASE_LEVEL < 7000 */
 #  endif
 #  if !defined (M_MACHINE)
 #    define M_MACHINE "Cray"
@@ -1580,7 +1874,6 @@ MAKE = make
 #define M_MACHINE "Siemens MX500"
 #define M_OS "SINIX V5.2x"
 #define USG
-
 #define HAVE_GETCWD
 #define VOID_SIGHANDLER
 #define HAVE_STRERROR
@@ -1636,7 +1929,7 @@ MAKE = make
 /* **************************************************************** */
 
 /* Use this entry for your machine if it isn't represented here.  It
-   is loosely based on a Vax running Bsd. */
+   is loosely based on a Vax running 4.3 Bsd. */
 
 #if !defined (M_MACHINE)
 #  define UNKNOWN_MACHINE
@@ -1682,38 +1975,55 @@ MAKE = make
 
 /* Define HAVE_GETGROUPS if your OS allows you to be in multiple
    groups simultaneously by supporting the `getgroups' system call. */
-/* #define HAVE_GETGROUPS */
+#  define HAVE_GETGROUPS
 
 /* Define SYSDEP_CFLAGS to be the flags to cc that make your compiler
    work.  For example, `-ma' on the RT makes alloca () work. */
 /* This is a summary of the semi-machine-independent definitions that
    can go into SYSDEP_CFLAGS:
 
-	USGr4	-	The machine is running SVR4
-	USGr4_2 -	The machine is running SVR4.2
-	USG	-	The machine is running some sort of System V Unix
 	AFS	-	The Andrew File System is being used
-	BSD_GETPGRP -	getpgrp(2) takes a pid argument, a la 4.3 BSD
-	TERMIOS_MISSING - the termios(3) functions are not present or don't
-			  work, even though _POSIX_VERSION is defined
-	PGRP_PIPE -	Requires parent-child synchronization via pipes to
-			make job control work right
-	HAVE_UID_T -	Definitions for uid_t and gid_t are in <sys/types.h>
-	HAVE_GETDTABLESIZE - getdtablesize(2) exists and works correctly
-	HAVE_SETDTABLESIZE - setdtablesize(2) exists and works correctly
-	HAVE_RESOURCE -	<sys/resource.h> and [gs]rlimit exist and work
-	MKFIFO_MISSING - named pipes do not work or mkfifo(3) is missing
-	REVERSED_SETVBUF_ARGS - brain-damaged implementation of setvbuf that
-				has args 2 and 3 reversed from the SVID and
-				ANSI standard
-	NO_DEV_TTY_JOB_CONTROL - system can't do job control on /dev/tty
+	AFS_CREATE_BUG - AFS has a bug with file creation if O_CREAT is
+			 specified
 	BROKEN_SIGSUSPEND - sigsuspend(2) does not work to wake up processes
 			    on SIGCHLD
+	BSD_GETPGRP -	getpgrp(2) takes a pid argument, a la 4.3 BSD
 	HAVE_BCOPY -	bcopy(3) exists and works as in BSD
+	HAVE_GETDTABLESIZE - getdtablesize(2) exists and works correctly
+	HAVE_GETHOSTNAME - gethostname(2) or gethostname(3) is present and
+			   works as in BSD
 	HAVE_GETPW_DECLS - USG machines with the getpw* functions defined in
 			   <pwd.h> that cannot handle redefinitions in the
 			   bash source
+	HAVE_RESOURCE -	<sys/resource.h> and [gs]rlimit exist and work
+	HAVE_SETDTABLESIZE - setdtablesize(2) exists and works correctly
+	HAVE_SOCKETS - this system has BSD sockets added to a System V base
+	HAVE_UID_T -	Definitions for uid_t and gid_t are in <sys/types.h>
+	INT_GROUPS_ARRAY - the second argument to getgroups(3) is an array
+			   of integers
+	MEMMOVE_MISSING - the system does not have memmove(3)
+	MKFIFO_MISSING - named pipes do not work or mkfifo(3) is missing
+	NEED_SYS_PARAM - MAXPATHLEN is defined in <sys/param.h>.  This is
+			 used by maxpath.h
+	NO_DEV_TTY_JOB_CONTROL - system can't do job control on /dev/tty
+	NO_SBRK_DECL - don't declare sbrk as extern char *sbrk() in
+		       lib/malloc/malloc.c
 	OPENDIR_NOT_ROBUST - opendir(3) allows you to open non-directory files
+	PGRP_PIPE -	Requires parent-child synchronization via pipes to
+			make job control work right
+	REVERSED_SETVBUF_ARGS - brain-damaged implementation of setvbuf that
+				has args 2 and 3 reversed from the SVID and
+				ANSI standard
+	RLOGIN_PGRP_BUG - processes started by rlogind have a process group
+			  of 0
+		     	TERMIOS_LDISC - system has a c_line line discipline member in struct
+			termios
+	TERMIOS_MISSING - the termios(3) functions are not present or don't
+			  work, even though _POSIX_VERSION is defined
+	USG	-	The machine is running some sort of System V Unix
+	USGr3	-	The machine is running SVR3.x
+	USGr4	-	The machine is running SVR4
+	USGr4_2 -	The machine is running SVR4.2
 */
 #  define SYSDEP_CFLAGS
 
@@ -1721,6 +2031,11 @@ MAKE = make
    in the C library, or a macro in a header file. */
 /* #define HAVE_STRERROR */
 
+/* Define HAVE_STRCASECMPif your system supplies definitions for the caseless
+   string comparison functions strcasecmp and strncasemp in libc or one of
+   the system header files. */
+/* #define HAVE_STRCASECMP */
+
 /* Define HAVE_DIRENT if you have the dirent library and a definition of
    struct dirent.  If not, the BSD directory reading library and struct
    direct are assumed. */
